#Licensed Materials - Property of IBM 
#Rational ClearQuest 
#Copyright IBM Corp. 2008 All Rights Reserved 

package cqcli::controlcmd;

use 5.008006;
use strict;
use warnings;

require Exporter;



use Error qw(:try);
use cqcli::Exception;
use Inline::Java qw(caught cast) ;
use cqcli::Util;
use cqcli::IOHandler;
use cqcli::commonservice;
use Text::ParseWords;
use Getopt::Long 2.34;


our @ISA = qw(Exporter);


our %EXPORT_TAGS = ( 'all' => [ qw( ) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(login logout  cqclisetlog status);

our $VERSION = '0.01';

#######################################################################
our %RepetitionCount = ();
our $isLoginOpt = undef;
our $CliSession = undef;
our %CmdInfo = ( 'cmdstring' => undef, 'cmdname'  => "login");
our %CmdLogin = ('database'	=>	undef,'schema'=>  undef,
			 'user'		=>	undef,'password'=>  undef);
our $BLANK_MARKER_SET = 0;

#########################################################################

#########handle getopt warnings begin########################

sub InitParser
{
	my @tmpARGV = @_;
	@ARGV = undef;
	%RepetitionCount = ();
	$BLANK_MARKER_SET = 0;
	%CmdInfo = ('cmdstring' => undef, 'cmdname'  => "login");
	%CmdLogin =('database'	=>undef,'schema'=>  undef,
				'user'	=>	undef,'password'=>  undef);
	$CmdInfo{'cmdstring'} =[@tmpARGV];
	($CmdInfo{'cmdname'}  , @ARGV ) = @{$CmdInfo{'cmdstring'}};
	$CliSession = GetSession();
	Getopt::Long::Configure('default');
	return 1;
}

sub CleanUpParser()
{
	%RepetitionCount = ();
	$BLANK_MARKER_SET = 0;
	%CmdInfo = ('cmdstring' => undef, 'cmdname'  => "edit_record");
	%CmdLogin =('database'	=>undef,'schema'=>  undef,
				'user'	=>	undef,'password'=>  undef);
	$CmdInfo{'cmdstring'} = undef;
	$CmdInfo{'cmdname'} =undef;
	$CliSession = undef;
	Getopt::Long::Configure('default');
}

sub handle_login_data
{
	my ($opt , $val) = @_;
	my $msg = undef;
	$isLoginOpt = 1;

	if($RepetitionCount{$opt}) {
		ThrowSyntaxErrorException("CQCLI_Repeat_Option","-$opt", $CmdInfo{'cmdname'} );
	}else {
		$RepetitionCount{$opt} = 1;
	}

	if( ($_[0] eq "database") && ($val)	){
		$CmdLogin{'database'} = $val;
	}elsif(($_[0] eq "user") && ($val)) {
		$CmdLogin{'user'} = $val;
	}elsif(($_[0] eq "password") && ($val)) {
		$CmdLogin{'password'} = $val;
	}elsif(($_[0] eq "password") && ($val eq "")){
		$CmdLogin{'password'} = "";
		$BLANK_MARKER_SET = 1 ;
	}
	elsif((($_[0] eq "schema") || ($_[0] eq "master")) && ($val) ) {
		$CmdLogin{'schema'} = $val; 
	}else{
		ThrowSyntaxErrorException("CQCLI_Missing_Value","-$opt",$CmdInfo{'cmdname'});
	}
	return 1;
}

sub handle_help_usage
{
	my $opt = $_[0];
	if($opt eq "help")	{
		ThrowUsageException($CmdInfo{'cmdname'},$opt);
	}
	else{
		ThrowUsageException($CmdInfo{'cmdname'},$opt);
	}
	return 1;
}

##############################################################################

sub parseForHelpUsage()
{
	my $ret = undef;
	my $helpRules = {'help|h|?'	=>	\&handle_help_usage	,'man'	=>	\&handle_help_usage	};
	Getopt::Long::Configure('autoabbrev','ignore_case','permute','pass_through');
	$ret = GetOptions( %{$helpRules} );
	Getopt::Long::Configure('default');

	return 1;
}

sub parseForLoginCredentials 
{
	my $ret = undef;
	my $CommonRules = {
		'database|dbs|d:s'					=>  \&handle_login_data			,
		'user|u:s'							=>	\&handle_login_data			,
		'schema|s|m|master:s'				=>	\&handle_login_data			,
		'password|p:s'						=>	\&handle_login_data			,
	};
	Getopt::Long::Configure( 'autoabbrev','ignore_case','permute','pass_through');
	$ret = GetOptions( %{$CommonRules} );
	Getopt::Long::Configure('default');
	return 1;
}


sub parseForUnknownOptions()
{
	if($#ARGV >=0)
	{
		ThrowSyntaxErrorException('CQCLI_Err_UnrecognizeOpt',@ARGV);
	}
	return 1;
}

##############################################################################################
sub ValidateSessionInfo()
{
	
	if($isLoginOpt){
		ValidateLoginCredentials();
		return;
	}elsif($CliSession){
		return;
	}else{
		ValidateLoginCredentials();
		return;
	}

}
sub ValidateLoginCredentials()
{
	if(!($CmdLogin{'user'})){
		$CmdLogin{'user'} = GetUserName();
	}
	if(!($CmdLogin{'password'}) && $BLANK_MARKER_SET == 0){
		$CmdLogin{'password'} = GetPassword();
	}
	if(!($CmdLogin{'database'})){
		$CmdLogin{'database'} = GetDBName();
	}
	if(!($CmdLogin{'schema'})){
		$CmdLogin{'schema'} = GetSchemaName();
	}
	$CliSession=CreateSession(%CmdLogin);
	$CliSession->getDefaultRecordType();
	# is not a command scope session
	SetSessionScope(0);
}

#######################################################################################

sub Parse()
{
	try {
		parseForHelpUsage();
		parseForLoginCredentials();
		parseForUnknownOptions();
		
	}
	catch cqcli::Exception with {
		my $ex = shift;		
		throw $ex;
	}
	otherwise {
		if (caught("java.lang.Exception")){
			ThrowJavaException($@,"CQCLI_ErrJava_ParserUnknown");
		}
		else{
			ThrowParseException('CQCLI_Err_ParserUnknown',$@);
		}
		
    }
	finally {
		Getopt::Long::Configure('default');
	};
}

sub Validate()
{
	try {
		ValidateSessionInfo();
	}
	catch cqcli::Exception with {
		my $ex = shift;		
		throw $ex;
	}
	otherwise {
		if (caught("java.lang.Exception")){
			ThrowJavaException($@,"CQCLI_ErrJava_ValidationUnknown");
		}
		else{
			ThrowValidationException('CQCLI_Err_ValidationUnknown',$@);
		}		
    }
	finally {	};

}

sub ParseAndValidateLogin
{
	my @tmpCmd = @_;
	try {
		InitParser(@tmpCmd);
		Parse();
		Validate();
	}
	finally {
		CleanUpParser();
	};
}

sub login
{
	my @tmpArray = @_;
	ParseAndValidateLogin(@tmpArray);
	return 1;
}

sub logout
{
	my ($cmd, @option) = @_;

	if($#option >=0){
		ThrowSyntaxErrorException('CQCLI_Err_UnrecognizeOpt',@option);
	}
	if(!(IsInteractiveMode)) {
		DestroySession();
	}elsif(WantToExit('CQCLI_Prompt_Exit',"cqtool")) {
			exit 0;
	}
	return 1;
}


sub status
{
	my ($cmd, @option) = @_;

	if($#option >=0){
		ThrowSyntaxErrorException('CQCLI_Err_UnrecognizeOpt',@option);
	}
	my $Session = GetSession();
	my ($user,$userdb,$schema,$duration) = (undef,undef,undef,undef);
	if($Session){
		$user = $Session->getLoginID();
		$userdb = $Session->getLoginDatabase();
		$schema = $Session->getLoginSchema();
		$duration = $Session->getLoginDuration();
		CLIPrintIdOutput('CQCLI_Login_Info',$user,$userdb,$schema,$duration);
		return 1;
	}else {

		CLIPrintIdOutput('CQCLI_Invalid_NoSession');
		return undef;
	}

}

sub cqclisetlog
{
	my $level = $_[1];
	
	if($level =~/^[\d]+\b/) {
		$ENV{CQCLI_ERR_LEVEL} = $level;
	}else {
		ThrowSyntaxErrorException('CQCLI_Err_IllegalValue',$level);
	}

	return 1;
}

1;
__END__